package Auth::Client;
use Moose;

use Data::Dumper;

use IO::Socket;

has 'sock_path' => (is => 'rw', isa => 'Str', required => 1);
has 'socket' => (is => 'rw', isa => 'Maybe[IO::Socket]');

sub set {
    my $self = shift;
    my %args = @_;
    my ($userid, $password) = @args{'userid', 'password'};

    $self->connect;
    $self->socket->print("\0SET\0$userid\0$password\0");
    my $response = $self->socket->getline;
    chomp $response;
    if ($response eq 'SET') {
        # warn "SET\n";
        return 1;
    }
    else {
        warn "Set failed\n";
        return 0;
    }
}
sub verify {
    my $self = shift;
    my %args = @_;
    # warn Dumper \%args;

    my ($userid, $password) = @args{'userid', 'password'};

    # warn "userid: $userid; password: $password\n";
    $self->connect;
    my $response;
    eval {
        $self->socket->print("\0VERIFY\0$userid\0$password\0");
        $response = $self->socket->getline;
    };
    die "Can't connect to server\n" if $@;
    chomp $response;
    if ($response eq 'OK') {
        # warn "OK\n";
        return 1;
    }
    else {
        # warn "Verify failed\n";
        return 0;
    }
}
sub delete {
    my $self = shift;
    my %args = @_;
    my $userid = $args{userid};
    $self->connect;
    my $response;
    eval {
        $self->socket->print("\0DELETE\0$userid\0NULL\0");
        $response = $self->socket->getline;
    };
    die "Can't connect to server\n" if $@;
    chomp $response;
    if ($response eq 'OK') {
        return 1;
    }
    else {
        return 0;
    }
}
sub connect {
    my $self = shift;
    unless (
        (defined $self->socket)
        && ($self->socket->connected)
    ) {
        my $socket = IO::Socket::UNIX->new(
            Type => SOCK_STREAM,
            Peer => $self->sock_path,
        );
        if (defined $socket) {
            $self->socket($socket);
        }
        else {
            die sprintf "Can't open connection to %s\n", $self->sock_path;
        }
    }
}

no Moose;
__PACKAGE__->meta->make_immutable;
